using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

namespace KollisionerMedPixlar
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        //Vr grafik
        Texture2D ship, ship_rect;
        Texture2D meteor, meteor_rect;
        Texture2D red;
        Texture2D hit;

        //Positioner
        double playerX, playerY, meteorX = 300, meteorY = 200;

        //verlappande rektangel
        Rectangle kollision;

        //Kollision?
        bool bHit = false;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";

            graphics.PreferredBackBufferWidth = 640;
            graphics.PreferredBackBufferHeight = 480;
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            hit = Content.Load<Texture2D>("hit");
            ship = Content.Load<Texture2D>("ship");
            meteor = Content.Load<Texture2D>("meteor");
            ship_rect = Content.Load<Texture2D>("ship_rect");
            meteor_rect = Content.Load<Texture2D>("meteor_rect");
            red = Content.Load<Texture2D>("red");
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            MouseState m = Mouse.GetState();
            playerX = m.X;
            playerY = m.Y;

            Rectangle playerBox = new Rectangle((int)playerX, (int)playerY, 48, 60);
            Rectangle meteorBox = new Rectangle((int)meteorX, (int)meteorY, 110, 110);

            //verlappar vi?
            kollision = Intersection(playerBox, meteorBox);

            if (kollision.Width > 0 && kollision.Height > 0)
            {
                Rectangle r1 = Normalize(playerBox, kollision);
                Rectangle r2 = Normalize(meteorBox, kollision);
                bHit = TestCollision(ship, r1, meteor, r2);
            }
            else
                bHit = false;

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            spriteBatch.Begin();
            spriteBatch.Draw(meteor, new Rectangle((int)meteorX, (int)meteorY, 110, 110), Color.White);
            spriteBatch.Draw(ship, new Rectangle((int)playerX, (int)playerY, 48, 60), Color.White);
            spriteBatch.Draw(meteor_rect, new Rectangle((int)meteorX, (int)meteorY, 110, 110), Color.White);
            spriteBatch.Draw(ship_rect, new Rectangle((int)playerX, (int)playerY, 48, 60), Color.White);

            //Har vi en rd rektangel?
            if (kollision.Width > 0 && kollision.Width > 0)
            {
                spriteBatch.Draw(red, kollision, new Rectangle(0, 0, kollision.Width, kollision.Height), Color.White);
            }
            //Har vi en kollision?
            if (bHit)
            {
                spriteBatch.Draw(hit, new Rectangle(150, 10, 150, 50), Color.White);
            }
            spriteBatch.End();


            base.Draw(gameTime);
        }

        public static Rectangle Intersection(Rectangle r1, Rectangle r2)
        {
            int x1 = Math.Max(r1.Left, r2.Left);
            int y1 = Math.Max(r1.Top, r2.Top);
            int x2 = Math.Min(r1.Right, r2.Right);
            int y2 = Math.Min(r1.Bottom, r2.Bottom);

            if ((x2 >= x1) && (y2 >= y1))
            {
                return new Rectangle(x1, y1, x2 - x1, y2 - y1);
            }
            return Rectangle.Empty;
        }

        public static Rectangle Normalize(Rectangle reference, Rectangle overlap)
        {
            //Rkna ut en rektangel som kan anvndas relativt till referensrektangeln
            return new Rectangle(
              overlap.X - reference.X,
              overlap.Y - reference.Y,
              overlap.Width,
              overlap.Height);
        }

        public static bool TestCollision(Texture2D t1, Rectangle r1, Texture2D t2, Rectangle r2)
        {
            //Berkna hur mnga pixlar som finns i omrdet som ska underskas
            int pixelCount = r1.Width * r1.Height;
            uint[] texture1Pixels = new uint[pixelCount];
            uint[] texture2Pixels = new uint[pixelCount];

            //Kopiera ut pixlarna frn bda omrdena
            t1.GetData(0, r1, texture1Pixels, 0, pixelCount);
            t2.GetData(0, r2, texture2Pixels, 0, pixelCount);

            //Jmfr om vi har ngra pixlar som verlappar varandra i omrdena
            for (int i = 0; i < pixelCount; ++i)
            {
                if (((texture1Pixels[i] & 0xff000000) > 0) && ((texture2Pixels[i] & 0xff000000) > 0))
                {
                    return true;
                }
            }
            return false;
        }
    }
}
